// If this code works, it was written by Svetlio Kapralov. If not, I don't know who wrote it.
package net.playerfinder.jsf.components.image;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.SocketAddress;
import java.net.URL;
import java.net.URLConnection;
import java.util.List;
import java.util.Map;

import javax.faces.application.ResourceHandler;
import javax.faces.component.UIComponent;
import javax.faces.component.behavior.ClientBehavior;
import javax.faces.component.behavior.ClientBehaviorContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.ConverterException;
import javax.faces.render.FacesRenderer;
import javax.faces.render.Renderer;
import javax.imageio.ImageIO;
import javax.servlet.ServletContext;

import net.playerfinder.jsf.components.common.Utils;

@FacesRenderer(componentFamily = UIImageCropper.COMPONENT_FAMILY, rendererType = UIImageCropper.DEFAULT_RENDERER)
public class UIImageCropperRenderer extends Renderer {

	private static String HIDDEN_INPUT = "_hiddenimagecropperfield";

	@Override
	public void decode(FacesContext context, UIComponent component) {
		UIImageCropper cropper = (UIImageCropper) component;
		Map<String, String> params = context.getExternalContext()
				.getRequestParameterMap();
		String coordsParam = cropper.getClientId(context) + HIDDEN_INPUT;

		if (params.containsKey(coordsParam)) {
			cropper.setSubmittedValue(params.get(coordsParam));
		}

		Utils.decodeBehaviors(context, component);
	}

	@Override
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		UIImageCropper cropper = (UIImageCropper) component;
		ResponseWriter writer = context.getResponseWriter();
		writer.startElement("span", null);
		encodeMarkup(context, cropper);
		encodeScript(context, cropper);
		writer.endElement("span");
	}

	private void encodeScript(FacesContext context, UIImageCropper cropper)
			throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = cropper.getClientId(context);

		writer.startElement("script", null);
		String updatehiddenInput = "updatehiddenInput_"
				+ clientId.replace(":", "_");

		ClientBehaviorContext behaviorContext = ClientBehaviorContext
				.createClientBehaviorContext(context, cropper, "crop",
						clientId, null);
		Map<String, List<ClientBehavior>> behaviors = cropper
				.getClientBehaviors();
		String cropajax = null;
		if (behaviors.containsKey("crop")) {
			cropajax = behaviors.get("crop").get(0).getScript(behaviorContext)
					.replace("event", "c.onSelect")
					+ ";";
		}

		writer.write("function " + updatehiddenInput + "(c) { "
				+ "document.getElementById('" + clientId + HIDDEN_INPUT
				+ "').value = '' + c.x + '_' + c.y + '_' + c.w + '_' + c.h; \n"
				+ cropajax + "};\n");

		writer.write("pf_jQuery(function($) { " + "$('#"
				+ Utils.escapeIdForJQuery(clientId + "_image") + "').Jcrop( { ");

		writer.write("onSelect: " + updatehiddenInput + ",");
		if (cropper.getMinSize() != null)
			writer.write("minSize:[" + cropper.getMinSize() + "],");
		if (cropper.getMaxSize() != null)
			writer.write("maxSize:[" + cropper.getMaxSize() + "],");
		if (cropper.getBackgroundColor() != null)
			writer.write("bgColor:'" + cropper.getBackgroundColor() + "',");
		if (cropper.getAspectRatio() != Double.MIN_VALUE)
			writer.write("aspectRatio:" + cropper.getAspectRatio() + ",");

		writer.write("allowSelect:" + cropper.isAllowSelect() + ",");
		writer.write("allowMove:" + cropper.isAllowMove() + ",");
		writer.write("allowResize:" + cropper.isAllowResize() + ",");

		String croppedCoords = getCropperCoords(context, cropper);
		if (!"".equals(croppedCoords)) {
			String[] cropCoords = getCropperCoords(context, cropper).split(",");
			int x = Integer.parseInt(cropCoords[0]);
			int y = Integer.parseInt(cropCoords[1]);
			int w = Integer.parseInt(cropCoords[2]);
			int h = Integer.parseInt(cropCoords[3]);
			int x2 = x + w;
			int y2 = y + h;
			String select = x + "," + y + "," + x2 + "," + y2;
			// Initial crop area
			writer.write("setSelect:[" + select + "],");
		}

		writer.write("bgOpacity:" + cropper.getBackgroundOpacity());
		writer.write("}); });");
		writer.endElement("script");
	}

	private String getCropperCoords(FacesContext context, UIImageCropper cropper) {
		String result = "";
		if (cropper.getValue() != null) {
			CroppedImage croppedImage = (CroppedImage) cropper.getValue();

			int x = croppedImage.getLeft();
			int y = croppedImage.getTop();
			int w = croppedImage.getWidth();
			int h = croppedImage.getHeight();

			result = x + "," + y + "," + w + "," + h;

		} else if (cropper.getInitialCoords() != null
				&& !"".equals(cropper.getInitialCoords())) {
			String[] cropCoords = cropper.getInitialCoords().split(",");
			int x = Integer.parseInt(cropCoords[0]);
			int y = Integer.parseInt(cropCoords[1]);
			int w = Integer.parseInt(cropCoords[2]);
			int h = Integer.parseInt(cropCoords[3]);

			try {
				// fix wrong initial coordinates
				BufferedImage outputImage = getSourceImage(context, cropper);
				if (outputImage.getWidth() < w)
					w = outputImage.getWidth();
				if (outputImage.getHeight() < h)
					h = outputImage.getHeight();
				if (x > w)
					x = 0;
				if (y > h)
					y = 0;
				if (x + w > outputImage.getWidth())
					w = outputImage.getWidth() - x;
				if (y + h > outputImage.getHeight())
					h = outputImage.getHeight() - y;
			} catch (IOException e) {
				e.printStackTrace();
			}

			result = x + "," + y + "," + w + "," + h;
		}
		return result;
	}

	private void encodeMarkup(FacesContext context, UIImageCropper cropper)
			throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		String clientId = cropper.getClientId(context);
		String hiddenInputId = clientId + HIDDEN_INPUT;

		writer.startElement("div", cropper);
		writer.writeAttribute("id", clientId, null);

		String alt = cropper.getImageAltText() == null ? "" : cropper
				.getImageAltText();

		writer.startElement("img", null);
		writer.writeAttribute("id", clientId + "_image", null);
		writer.writeAttribute("name", clientId + "_image", null);
		writer.writeAttribute("alt", alt, null);
		writer.writeAttribute("src",
				getResourceURL(context, cropper.getImageSrc()), null);
		writer.endElement("img");

		writer.startElement("input", null);
		writer.writeAttribute("type", "hidden", null);
		writer.writeAttribute("id", hiddenInputId, null);
		writer.writeAttribute("name", hiddenInputId, null);
		writer.writeAttribute("value",
				getCropperCoords(context, cropper).replace(",", "_"), null);
		writer.endElement("input");

		writer.endElement("div");
	}

	private String getResourceURL(FacesContext facesContext, String value) {
		if (value == null || "".equals(value))
			return "";
		if (value.contains(ResourceHandler.RESOURCE_IDENTIFIER)) {
			return value;
		} else {
			String url = facesContext.getApplication().getViewHandler()
					.getResourceURL(facesContext, value);

			return facesContext.getExternalContext().encodeResourceURL(url);
		}
	}

	@Override
	public Object getConvertedValue(FacesContext context,
			UIComponent component, Object submittedValue)
			throws ConverterException {
		String coords = (String) submittedValue;
		if (coords == null || "".equals(coords)) {
			return null;
		}

		UIImageCropper cropper = (UIImageCropper) component;
		String[] cropCoords = coords.split("_");
		String format = getImageFormat(cropper.getImageSrc());

		int x = Integer.parseInt(cropCoords[0]);
		int y = Integer.parseInt(cropCoords[1]);
		int w = Integer.parseInt(cropCoords[2]);
		int h = Integer.parseInt(cropCoords[3]);

		try {
			BufferedImage outputImage = getSourceImage(context, cropper);

			if (x + w > outputImage.getWidth())
				w = outputImage.getWidth() - x;
			if (y + h > outputImage.getHeight())
				h = outputImage.getHeight() - y;
			
			BufferedImage cropped = outputImage.getSubimage(x, y, w, h);

			ByteArrayOutputStream croppedOutImage = new ByteArrayOutputStream();
			ImageIO.write(cropped, format, croppedOutImage);

			return new CroppedImage(cropper.getImageSrc(),
					croppedOutImage.toByteArray(), x, y, w, h);

		} catch (IOException e) {
			throw new ConverterException(e);
		}
	}

	private String getImageFormat(String path) {
		String[] pathTokens = path.split("\\.");

		return pathTokens[pathTokens.length - 1];
	}

	private boolean isHttpImage(UIImageCropper cropper) {
		return cropper.getImageSrc().toLowerCase().startsWith("http");
	}

	private BufferedImage getSourceImage(FacesContext context,
			UIImageCropper cropper) throws IOException {
		BufferedImage outputImage = null;
		boolean isHttp = isHttpImage(cropper);

		if (isHttp) {
			URL url = new URL(cropper.getImageSrc());
			
			// This is where you'd define the proxy's host name and port.
			int port = 80; //default
			if (url.getPort() != -1)
				port = url.getPort();
				
            SocketAddress address = new InetSocketAddress(url.getHost(), port);
            
            // Create an HTTP Proxy using the above SocketAddress.
            Proxy proxy = new Proxy(Proxy.Type.HTTP, address);
            
            // Open a connection to the URL using the proxy information.
            URLConnection conn = url.openConnection(proxy);
            InputStream inStream = conn.getInputStream();
            
            // outputImage  = ImageIO.read(url);
            // Use the InputStream flavor of ImageIO.read() instead.
            outputImage = ImageIO.read(inStream);
			
		} else {
			ServletContext servletContext = (ServletContext) context
					.getExternalContext().getContext();
			String imagePath = servletContext.getRealPath("")
					+ cropper.getImageSrc();

			outputImage = ImageIO.read(new File(imagePath));
		}

		return outputImage;
	}
}